# AutoSave.pl
# Auto-save v1.3b plugin for Movable Type
# http://www.chipple.net/mt/AutoSave/
# by Patrick Benny http://www.chipple.net/
# Free to use. All rights reserved.
# This code cannot be redistributed without permission from the author. 
#
# Version history
#   1.3.1 - 2007/5/24 - Fixed Gecko 1.9 bug (Firefox 3)
#   1.3a  - 2007/4/12 - Fixed Safari bug
#   1.2   - 2006/9/25 - Reset entry date/time when using "Save" button in bookmarklet
#                       "Saved at XX:XX:XX" now displays server time
#                       Fixed minor bug causing potential warning when using default settings (Thanks to drry)
#   1.1   - 2006/9/13 - Fixed bugs relating to bookmarklet
#   1.0.2 - 2006/9/12 - Added message in bookmarklet asking to select weblog
#   1.0.1 - 2006/9/11 - Fixed minor risk of auto-save after user clicked "Save"
#   1.0   - 2006/9/8  - First public release


package AutoSave;

use strict;

use warnings;

use base 'MT::App::CMS';
use MT::Util qw( offset_time_list );


sub init {
	my $app = shift;
	$app->SUPER::init(@_) or return;
	$app->add_methods(
		'auto_save_entry' => \&auto_save_entry,
		'bm_save_entry'   => \&bm_save_entry,
	);
	$app->{default_mode} = 'default';
	$app;
}


sub default {
	print "Content-Type: text/html\n\n";
	print "Default mode works\n";
}



####################
### MAIN METHODS ###
####################

# Save an entry, forcing "unpublished" status
sub auto_save_entry {
    my $app = shift;
    my $q = $app->param;
    $q->param(status => MT::Entry::HOLD());
    MT::App::CMS::save_entry($app);

    # If new entry, extract ID from redirect URL
    my $id = $q->param('id');
    $id = "$1" if !$id && $app->{redirect} =~ m/&id=(\d+)/;

    # Output script to browser
    my @ts = offset_time_list(time, $q->param('blog_id'));
    my $ts = sprintf '%02d:%02d:%02d', @ts[2,1,0];
    print('Content-Type: text/html

<html>
<head>
<script type="text/javascript">
var f,els,el;
if (  parent
   && (f = parent.document.forms["entry_form"])) {
  // Add "id" element to form
  if (!(els = f.elements)["id"]) {
    el = parent.document.createElement("input");
    el.type = "hidden";
    el.name = "id";
    el.value = "'.$id.'";
    els[els.length] = f.insertBefore(el,f.firstChild);
  }
  // Unlock form
  parent.f_AS_LockForm(false);
  // Update status
  var d = new Date();
  parent.f_AS_UpdateStatus("Saved at '.$ts.'");
}

// Pads with zeroes to 2 digits
function f_Pad (i) {
  var s = i.toString();
  while (s.length < 2) s = "0"+i;
  return s;
}
</script>
</head>
<body>
<p>Saved entry '.$id.'.</p>
<p>Redirect URL: '.$app->{redirect}.'.</p>
</body>
</html>');

    # Suppress further output
    $app->{redirect} = '';
    $app->{no_print_body} = 1;
}

# Regular save_entry except that we force created_on to current time
sub bm_save_entry {
    my $app = shift;
    my $q = $app->param;
    my @ts = offset_time_list(time, $q->param('blog_id'));
    my $ts = sprintf '%04d%02d%02d%02d%02d%02d',
        $ts[5]+1900, $ts[4]+1, @ts[3,2,1,0];
    $q->param(created_on => $ts);
    MT::App::CMS::save_entry($app);
}

1;
